﻿using System;
using System.Diagnostics;
using System.Linq;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Threading;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace DisplayASimpleMapForWPF
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        DispatcherTimer timer;
        bool shouldSave;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void WpfMap_Loaded(object sender, RoutedEventArgs e)
        {
            wpfMap1.MapUnit = GeographyUnit.DecimalDegree;

            OsmWorldMapKitLayer osmWorldMapKitLayer = new OsmWorldMapKitLayer("Data Source=\\\\\\\\192.168.0.3\\\\Map Data\\\\WorldMapKit-4326-20151108-381061331.sqlite;Version=3;Read Only=True;", OsmWorldMapKitDatabaseType.Sqlite);
            osmWorldMapKitLayer.Open();

            LayerOverlay staticOverlay = new LayerOverlay();
            staticOverlay.TileType = TileType.SingleTile;
            staticOverlay.Layers.Add(osmWorldMapKitLayer);

            wpfMap1.Overlays.Add(staticOverlay);

            ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(@"..\..\Data\Countries02.shp");
            worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.Transparent, GeoColor.FromArgb(100, GeoColor.SimpleColors.Green));

            LayerOverlay dynamicOverlay = new LayerOverlay();
            dynamicOverlay.Layers.Add("WorldLayer", worldLayer);
            dynamicOverlay.TileType = TileType.SingleTile;
            wpfMap1.Overlays.Add(dynamicOverlay);

            wpfMap1.CurrentExtent = new RectangleShape(-133.2515625, 89.2484375, 126.9046875, -88.290625);

            foreach (Overlay overlay in wpfMap1.Overlays.OfType<TileOverlay>()
                                .Where((to => to.IsVisible)))
            {
                TileOverlay tileOverlay = overlay as TileOverlay;
                tileOverlay.DrawTilesProgressChanged += new System.EventHandler<DrawTilesProgressChangedTileOverlayEventArgs>(tileOverlay_DrawTilesProgressChanged);
            }

            wpfMap1.CurrentExtentChanged += wpfMap1_CurrentExtentChanged;
            wpfMap1.Refresh();
        }

        void wpfMap1_CurrentExtentChanged(object sender, CurrentExtentChangedWpfMapEventArgs e)
        {
            shouldSave = true;
        }

        void tileOverlay_DrawTilesProgressChanged(object sender, DrawTilesProgressChangedTileOverlayEventArgs e)
        {
            var tileOverlays = wpfMap1.Overlays
                                .OfType<TileOverlay>()
                                .Where((to => to.IsVisible));

            bool isFinished = tileOverlays.All(o => CheckIsDrawn(o));

            if (timer == null)
            {
                timer = new DispatcherTimer();
                timer.Interval = TimeSpan.FromSeconds(1);
                timer.Tick += (s, a) =>
                {
                    timer.Stop();

                    if (shouldSave)
                    {
                        string id = Guid.NewGuid().ToString();
                        System.Drawing.Bitmap map = wpfMap1.GetBitmap(1000, 800);
                        map.Save("d:\\A" + id + ".png");
                        Process.Start("d:\\A" + id + ".png");
                        shouldSave = false;
                    }
                };
            }

            if (isFinished)
            {
                timer.Stop();
                timer.Start();
            }
        }

        private static bool CheckIsDrawn(TileOverlay tileOverlay)
        {
            bool isFinished = true;
            Canvas drawingCanvas = ((Canvas)tileOverlay.OverlayCanvas.Children[0]);
            if (drawingCanvas.Children.Count != 0)
            {
                foreach (ThinkGeo.MapSuite.WpfDesktopEdition.Tile tile in drawingCanvas.Children)
                {
                    if (!tile.IsOpened)
                    {
                        isFinished = false;
                        break;
                    }
                }
            }
            else isFinished = false;

            return isFinished;
        }
    }
}
